/*
 * PhotoUpload.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr.example;

import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import javax.imageio.ImageIO;
import org.jickr.*;

/**
 *
 * @author driscoll
 */
public class PhotoUpload {
    
    // Hardcoded directory, since I don't want to include the directory choosing
    // code into this - it'd bloat the code.
    public static final String DESTDIR = "/home/driscoll/Desktop/FlickrBackupPhotos";

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
        
        // We'll read the API Key and Shared Secret from a file, since
        // We don't want to check in our keys.
        // Normally, we'd use the next block of code and hardcode the keys instead
        
        BufferedReader br = new BufferedReader(new FileReader("keys"));
        String apiKey = br.readLine();
        Flickr.setApiKey(apiKey);
        String sharedSecret = br.readLine();
        Flickr.setSharedSecret(sharedSecret);
        
        /*
        //  The Following block is the ususal one to include in programs.
        String apiKey = "notavalidapikey";
        Flickr.setApiKey(apiKey);
         
        String sharedSecret = "notavalidsharedsecret";
        Flickr.setSharedSecret(sharedSecret);
         */
        
        User user = Auth.getDefaultAuthUser();
        if (user == null) throw new Exception("Login in a different app (like LoginTest)");
        Auth.setAuthContext(Auth.getDefaultAuthUser());
        
        // First, we'll get a photo, and use it to make a file.
        Photo origPhoto = new Photo("202653725");
        Photo changePhoto = new Photo("202654075");
        File t1 = File.createTempFile("img",".jpg");
        File t2 = File.createTempFile("img",".jpg");
        BufferedImage image = origPhoto.getImage(Photo.Size.THUMB);  // Small one, since we're not testing with a pro account :-)
        ImageIO.write(image,"jpg",t1);
        image = changePhoto.getImage(Photo.Size.THUMB);
        ImageIO.write(image,"jpg",t2);
        
        // Now, we'll upload the file
        Photo newPhoto = Photo.create(t1,"Copy of "+origPhoto.getTitle());
        System.out.println("Success! Photo ID is "+newPhoto.getID());
        
        // Change the photo to something else.
//        System.out.println("Go look at the photo - I'm about to change it.  Hit return to change photo.");
//        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
//        in.readLine();
//        
//        Photo modPhoto = newPhoto.replace(t2);
//        
//        System.out.println("Changed photo - Photo ID is "+modPhoto.getID());
//        
        // Clean up and delete the photo.
        System.out.println("Go look at the photo, I'm about to delete it.  Hit return to delete photo.");
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        in.readLine();

        newPhoto.delete();
        System.out.println("Photo deleted");
    }
}
