/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.descriptor;

import java.io.File;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * This class parses photo descriptors and the album descriptor and
 * creates an XML file of the entire gallery. This file is not used
 * by Atacama and is meant for potential future use.
 */
public class DescriptorParser {
	//Logger, Config and FileManager
	static private Config theConfig = null;
	static private Logger theLogger = null;
	static private FileManager theFileManager = null;

	/**
	 * Constructor
	 */
	public DescriptorParser() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * Parses the photo and album descriptors into an XML file
	 */
	public void parseGalleryToXml() {
		DescriptorReader descReader = new DescriptorReader();
		
		Vector gallery = null;
		try {
			gallery = descReader.readGallery();
		} catch(AtacamaException exception) {
			theLogger.logWarning("The gallery was not parsed to XML. " + exception.getMessage());
			return;
		}

		Document document = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.newDocument();

			Element galleryXml = (Element) document.createElement("gallery");
			document.appendChild(galleryXml);

			Element albumElement = null;
			Album album = null;
			Photo photo = null;
			Vector photos = null;
			for (int albumIndex = 0; albumIndex < gallery.size(); albumIndex++) {
				album = (Album) gallery.elementAt(albumIndex);
				albumElement = document.createElement("album");

				Element nameElement  = document.createElement("name");
				Element yearElement  = document.createElement("year");
				Element countryElement  = document.createElement("country");
				Element monthElement  = document.createElement("month");
				Element commentElement  = document.createElement("comment");
				Element photosElement  = document.createElement("photos");

				nameElement.appendChild(document.createTextNode(album.getAlbumName()));
				yearElement.appendChild(document.createTextNode(album.getYear()));
				countryElement.appendChild(document.createTextNode(album.getCountry()));
				monthElement.appendChild(document.createTextNode(album.getMonth()));
				commentElement.appendChild(document.createTextNode(album.getComment()));

				photos = album.getPhotos();
				for (int photoIndex = 0; photoIndex < photos.size(); photoIndex++) {
					photo = (Photo) photos.elementAt(photoIndex);
					Element photoElement = document.createElement("photo");

					Element photoNameElement  = document.createElement("name");
					Element fileSizeElement  = document.createElement("fileSize");
					Element filePathElement  = document.createElement("filePath");
					Element photoCommentElement  = document.createElement("comment");

					photoNameElement.appendChild(document.createTextNode(photo.getPhotoName()));
					fileSizeElement.appendChild(document.createTextNode(photo.getFileSize()));
					filePathElement.appendChild(document.createTextNode(photo.getPhotoFilePath()));
					photoCommentElement.appendChild(document.createTextNode(photo.getComment()));

					photoElement.appendChild(photoNameElement);
					photoElement.appendChild(fileSizeElement);
					photoElement.appendChild(filePathElement);
					photoElement.appendChild(photoCommentElement);

					photosElement.appendChild(photoElement);
				}

				albumElement.appendChild(nameElement);
				albumElement.appendChild(yearElement);
				albumElement.appendChild(countryElement);
				albumElement.appendChild(monthElement);
				albumElement.appendChild(commentElement);
				albumElement.appendChild(photosElement);

				galleryXml.appendChild(albumElement);
			}

		} catch (ParserConfigurationException pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();
		}

		//Write XML to file		
		String galleryFilePath = theConfig.getApplicationDir();
		galleryFilePath += theConfig.getString("descriptor.file.xml.gallery");
		
		try {
			theFileManager.writeXmlFile(document, new File(galleryFilePath));
		} catch(AtacamaException exception) {
			theLogger.logWarning("The gallery XML file was not written to file. " + exception.getMessage());
			return;
		}
	}
}
