/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.entity;

import java.util.Vector;

import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * Represents a photo album
 */
public class Album {

	//Constants
	private static final int NUM_OF_FIELDS = 7;
	
	//Class fields
	private String albumName = "";
	private String publish = "";
	private String privateOnly = "";
	private String year = "";
	private String country = "";
	private String month = "";
	private String comment = "";
	private Vector photos = null;
	
	/**
	 * Constructor that creates emtpy album
	 */
	public Album() {
		
	}

	/**
	 * Constructor that takes descriptor line
	 * @param descriptorLine Semi-colon seperated string to be parsed into fields
	 * @throws AtacamaException Exception thrown if parsing fails
	 */
	public Album(String descriptorLine) throws AtacamaException {
		parseDescriptorLine(descriptorLine);	
	}
	
	/**
	 * Compares all fields in objects
	 * @param album The album to be compared
	 * @return True if equal
	 */
	public boolean equals(Album album) {
		if(albumName != null
			&& publish != null
			&& privateOnly != null
			&& year != null
			&& country != null
			&& month != null
			&& comment != null
			&& photos != null
			&& albumName.equals(album.getAlbumName())
			&& year.equals(album.getYear())
			&& country.equals(album.getCountry())
			&& month.equals(album.getMonth())
			&& comment.equals(album.getComment())) {
				//All string fields are equal, now check the photo array
				if(photos != null 
					&& photos.size() == album.getPhotos().size()) {
						//Photo arrays have are not empty and have same lenght, compare elements
						Photo photo1 = null;
						Photo photo2 = null;
						for(int index = 0; index < photos.size();index++) {
							photo1 = (Photo) photos.elementAt(index);
							photo2 = (Photo) album.getPhotos().elementAt(index);
							
							if(!photo1.equals(photo2))
								return false;
							index++;
						}
					} else {
						return false;
					}

			} else {
				return false;
			}
		
		return true;
	}

	
	/**
	 * Parses a album descriptor line into the fields of a album object
	 * 
	 * @param descriptorLine The semi-colon separated string to be parsed
	 * @throws AtacamaException If parsing failes
	 */
	private void parseDescriptorLine(String descriptorLine) throws AtacamaException {
		if("".equals(descriptorLine) || descriptorLine == null) {
			throw new AtacamaException("Log descriptor line is empty, cannot create Photo");
		}
		
		String[] fields = descriptorLine.split(";", -1);
		
		if(fields.length != NUM_OF_FIELDS) {
			throw new AtacamaException(
				"Log descriptor line has " 
				+ fields.length
				+ " fields instead of "
				+ NUM_OF_FIELDS 
				+ " fields: '" 
				+ descriptorLine
				+ "'");
		}
		
		setAlbumName(fields[0]);
		setPublish(fields[1]);
		setPrivateOnly(fields[2]);
		setYear(fields[3]);
		setCountry(fields[4]);
		setMonth(fields[5]);
		setComment(fields[6]);
		setPhotos(new Vector());
	}
	
	/**
	 * Creates a semi-colon separated string with the album fields
	 * not including photos.
	 * 
	 * @return The semi-colon separated string.
	 */
	public String toString() {
		return
			getAlbumName()
			+ ";"
			+ getPublish()
			+ ";"
			+ getPrivateOnly()
			+ ";"
			+ getYear()
			+ ";"
			+ getCountry()
			+ ";"
			+ getMonth()
			+ ";"
			+ getComment();
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getComment() {
		return validateField(comment);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getCountry() {
		return validateField(country);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getAlbumName() {
		return validateField(albumName);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getMonth() {
		return validateField(month);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getPublish() {
		return validateField(publish);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getPrivateOnly() {
		return validateField(privateOnly);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getYear() {
		return validateField(year);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setComment(String string) {
		comment = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setCountry(String string) {
		country = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setAlbumName(String string) {
		albumName = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setMonth(String string) {
		month = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setPublish(String string) {
		publish = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setPrivateOnly(String string) {
		privateOnly = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setYear(String string) {
		year = validateField(string);
	}
	/**
	 * Gets the photos vector
	 * @return The vector
	 */
	public Vector getPhotos() {
		return photos;
	}

	/**
	 * Sets field
	 * @param vector The vector to be set
	 */
	public void setPhotos(Vector vector) {
		photos = vector;
	}

	/**
	 * Removes semi-colon characters and and returns the trimmed field
	 * 
	 * @param fieldValue
	 * @return The validated field
	 */
	private String validateField(String fieldValue) {
		if(fieldValue.indexOf(";") != -1) {
			fieldValue = fieldValue.replace(';', ':');
		}
		
		return fieldValue.trim();
	}

	/**
	 * Gets the primary photo of the album. If none of the photos in the album has the 
	 * primary property set, the first photo in the album is returned. If multiple
	 * primary properties are set, the first one is used.
	 * @return The primary photo
	 */
	public Photo getPrimaryPhoto() {
		Photo primaryPhoto = null;

		if(photos != null) {
			//For each photo in the album look for the primary property
			for(int photoIndex = 0; photoIndex < photos.size(); photoIndex++) {
				Photo tempPhoto = (Photo) photos.elementAt(photoIndex);
				if("true".equals(tempPhoto.getPrimary())) {
					//Primary property found, set primary photo
					primaryPhoto = tempPhoto;
					break;
				}
			}
			
			if(primaryPhoto == null) {
				//Primary not found, use first
				primaryPhoto = (Photo) photos.elementAt(0);
			}
		}
		
		return primaryPhoto;
	}
}
