/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.entity;

import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * Represents of a piece of media with descriptions of the file
 * and its album
 */
public class Photo {
	
	//Constants
	private static final int NUM_OF_FIELDS = 6;
	
	//Class fields
	private String photoName = "";
	private String publish = "";
	private String albumName = "";
	private String fileSize = "";
	private String primary = "";
	private String comment = "";
	private String photoFilePath = "";
	
	/**
	 * Constructor. Fields are parsed from a photo decriptor line.
	 * @param descriptorLine The semi-colon string to be parsed
	 * @throws AtacamaException
	 */
	public Photo(String descriptorLine) throws AtacamaException {
		parseDescriptorLine(descriptorLine);	
	}
	
	/**
	 * Constructor. 
	 */
	public Photo() {

	}

	/**
	 * Compares all fields in objects
	 * @param photo The photo to be compared
	 * @return True if equal
	 */
	public boolean equals(Photo photo) {
		if(photoName != null
			&& publish != null
			&& albumName != null
			&& fileSize != null
			&& primary != null
			&& comment != null
			&& photoFilePath != null
			&& photoName.equals(photo.getPhotoName())
			&& publish.equals(photo.getPublish())
			&& albumName.equals(photo.getAlbumName())
			&& fileSize.equals(photo.getFileSize())
			&& primary.equals(photo.getPrimary())
			&& comment.equals(photo.getComment())
			&& photoFilePath.equals(photo.getPhotoFilePath())) {
				return true;
			} else {
				return false;
			}
	}
	
	/**
	 * Parses a photo descriptor line into the fields of a Photo object
	 * 
	 * @param descriptorLine The semi-colon string to be parsed
	 * @throws AtacamaException If parsing fails
	 */
	private void parseDescriptorLine(String descriptorLine) throws AtacamaException {
		if("".equals(descriptorLine) || descriptorLine == null) {
			throw new AtacamaException("Log descriptor line is empty, cannot create Photo");
		}
		
		String[] fields = descriptorLine.split(";", -1);
		
		if(fields.length != NUM_OF_FIELDS) {
			throw new AtacamaException(
				"Log descriptor line has " 
				+ fields.length
				+ " fields instead of "
				+ NUM_OF_FIELDS 
				+ " fields: '" 
				+ descriptorLine
				+ "'");
		}
				
		setPhotoName(fields[0]);
		setPublish(fields[1]);
		setAlbumName(fields[2]);
		setFileSize(fields[3]);
		setPrimary(fields[4]);
		setComment(fields[5]);
		setPhotoFilePath("");
	}

   /**	
	* Creates a semi-colon separated string with the photo fields
	* 
	* @return The semi-colon separated string.
	*/
	public String toString() {
		return
			getPhotoName()
			+ ";"
			+ getPublish()
			+ ";"
			+ getAlbumName()
			+ ";"
			+ getFileSize()
			+ ";"
			+ getPrimary()
			+ ";"
			+ getComment();
	}
	
	/**
	 * Returns field
	 * @return The field value
	 */
	public String getFileSize() {
		return validateField(fileSize);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getAlbumName() {
		return validateField(albumName);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getPhotoName() {
		return validateField(photoName);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getPublish() {
		return validateField(publish);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setFileSize(String string) {
		fileSize = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setAlbumName(String string) {
		albumName = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setPhotoName(String string) {
		photoName = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setPublish(String string) {
		publish = validateField(string);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getPrimary() {
		return validateField(primary);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getComment() {
		return validateField(comment);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setPrimary(String string) {
		primary = validateField(string);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setComment(String string) {
		comment = validateField(string);
	}

	/**
	 * Returns field
	 * @return The field value
	 */
	public String getPhotoFilePath() {
		return validateField(photoFilePath);
	}

	/**
	 * Sets the field
	 * @param string The field value
	 */
	public void setPhotoFilePath(String string) {
		photoFilePath = validateField(string);
	}

	/**
	 * Removes semi-colon characters and and returns the trimmed field
	 * 
	 * @param fieldValue
	 * @return The validated field
	 */
	private String validateField(String fieldValue) {
		if(fieldValue.indexOf(";") != -1) {
			fieldValue = fieldValue.replaceAll(";", ":");
		}
		
		return fieldValue.trim();
	}

}
