/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Point;
import java.awt.ScrollPane;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseWheelEvent;
import java.io.File;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * This class displays the scroll pane containing all the albums in the
 * current album descriptor. It's responsible for adding, moving and removing
 * the albums in the scroll pane.
 */
public class AlbumScrollPane extends ScrollPane implements ActionListener {

	//Controller, Config, Logger and FileManager	
	private static Controller theController = null;
	private static Config theConfig = null;

	//Components
	public Vector publishBoxes = null;
	public Vector privateOnlyBoxes = null;
	public Vector albumNameFields = null;
	public Vector commentFields = null;
	public Vector yearFields = null;
	public Vector monthFields = null;
	public Vector countryFields = null;
	public Vector upButtons = null;
	public Vector removeButtons = null;
	public Vector downButtons = null;
	public Vector toTopButtons = null;
	public Vector insertNewButtons = null;
	public Vector toBottomButtons = null;

	/**
	 * The constructor
	 */
	public AlbumScrollPane() {
		//Get the controller, config, logger and file manager
		theController = Controller.getInstance();
		theConfig = Config.getInstance();		
	}

	/**
	 * Initialises the panel with all components
	 * @param descriptorPath The path to the album descriptor to be displayed
	 */
	public void init(String descriptorPath) {	
		//Might be refreshing, removel all first
		removeAll();
						
		//Get the albums for the specified descriptor
		Vector albums = null;
		albums = theController.readAlbumDescriptor(descriptorPath, true);
		
		//Create all indexed components
		createAlbumInfoVectors(albums);
		
		//Add rows panel to scroll pane
		add(initRowsPanel());
	}
	
	/**
	 * Initializes the rows panel containing all album rows
	 * 
	 * @return The panel with all rows
	 */
	private Panel initRowsPanel() {
		//Create new rows planel
		Panel rowsPanel = new Panel();

		//Set rowspanel to same number of rows as album vector
		GridLayout gridLayout = new GridLayout();
		gridLayout.setColumns(3);
		gridLayout.setRows(albumNameFields.size());
		rowsPanel.setLayout(gridLayout);
		
		//Add rows to panel for each album in descriptor		
		for (int index = 0; index < albumNameFields.size(); index++) {
			//Create up down buttons panel
			Panel upDownButtonsPanel = new Panel();
			upDownButtonsPanel.setLayout(new GridLayout(3, 1));
			upDownButtonsPanel.add((Button) toTopButtons.elementAt(index));
			upDownButtonsPanel.add((Button) upButtons.elementAt(index));
			upDownButtonsPanel.add((Button) insertNewButtons.elementAt(index));
			upDownButtonsPanel.add((Button) removeButtons.elementAt(index));
			upDownButtonsPanel.add((Button) toBottomButtons.elementAt(index));
			upDownButtonsPanel.add((Button) downButtons.elementAt(index));

			//Create add columns panel
			Panel columnsPanel = new Panel();
			columnsPanel.setLayout(new BorderLayout());
			columnsPanel.add(initAlbumInfoPanel(index), "Center");
			columnsPanel.add(upDownButtonsPanel, "East");

			//Alternate color of rows
			if(new Float(index).floatValue()/2 - (index/2) == 0.5) {
				columnsPanel.setBackground(Color.gray);
			}

			//Add columns panel to row panel
			rowsPanel.add(columnsPanel);			
		}

		return rowsPanel;
	}

	/**
	 * Creates all album info (name, month, year etc) components and
	 * inserts them into vectors for later retrieval.
	 * @param albums The vector of albums to create vectors from
	 */
	private void createAlbumInfoVectors(Vector albums) {
		//Instantiate vectors
		publishBoxes = new Vector(albums.size());
		privateOnlyBoxes = new Vector(albums.size());
		albumNameFields = new Vector(albums.size());
		commentFields = new Vector(albums.size());
		yearFields =  new Vector(albums.size());
		monthFields =  new Vector(albums.size());
		countryFields = new Vector(albums.size());
		upButtons = new Vector(albums.size());
		removeButtons = new Vector(albums.size());
		downButtons = new Vector(albums.size());
		toTopButtons = new Vector(albums.size());
		insertNewButtons = new Vector(albums.size());
		toBottomButtons = new Vector(albums.size());

		for(int index = 0; index < albums.size(); index++) {
			//Get corresponding album
			Album album = (Album) albums.elementAt(index);			
			
			//Insert row info at bottom
			addAlbumRowInfo(album, -1);			
		}
	}

	/**
	 * Creates a panel with the information for a single album
	 * 
	 * @param index The index of the album to create
	 * @return The album info panel to be inserted into a row panel
	 */
	private Panel initAlbumInfoPanel(int index) {
		//Create album name panel static components
		Label albumLabel = new Label("Album:       ");
		albumLabel.setFont(new Font("Arial", Font.BOLD, 12));

		//Create album name panel
		Panel albumNamePanel = new Panel();
		albumNamePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		albumNamePanel.add(albumLabel);
		albumNamePanel.add((TextField) albumNameFields.elementAt(index));
		albumNamePanel.add(new Label("    "));
		albumNamePanel.add((Checkbox) publishBoxes.elementAt(index));
		albumNamePanel.add((Checkbox) privateOnlyBoxes.elementAt(index));

		//Create album year, month and country static components
		Label yearLabel = new Label("Year:          ");
		yearLabel.setFont(new Font("Arial", Font.BOLD, 12));
		Label monthLabel = new Label("Month:");
		monthLabel.setFont(new Font("Arial", Font.BOLD, 12));
		Label countryLabel = new Label("Location:");
		countryLabel.setFont(new Font("Arial", Font.BOLD, 12));

		//Create album year, month and country panel
		Panel yearMonthCountryPanel = new Panel();
		yearMonthCountryPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		yearMonthCountryPanel.add(yearLabel);
		yearMonthCountryPanel.add((TextField) yearFields.elementAt(index));
		yearMonthCountryPanel.add(new Label("    "));
		yearMonthCountryPanel.add(monthLabel);
		yearMonthCountryPanel.add((TextField) monthFields.elementAt(index));
		yearMonthCountryPanel.add(new Label("    "));
		yearMonthCountryPanel.add(countryLabel);
		yearMonthCountryPanel.add((TextField) countryFields.elementAt(index));

		//Create comment text area panel static components
		Label commentLabel = new Label("Comment:");
		commentLabel.setFont(new Font("Arial", Font.BOLD, 12));

		//Create comment text area panel
		Panel albumCommentPanel = new Panel();
		albumCommentPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		albumCommentPanel.add(commentLabel);
		albumCommentPanel.add((TextField) commentFields.elementAt(index));

		//Create album info panel
		Panel albumInfoPanel = new Panel();
		albumInfoPanel.setLayout(new BorderLayout());
		albumInfoPanel.add(albumNamePanel, "North");
		albumInfoPanel.add(yearMonthCountryPanel, "Center");
		albumInfoPanel.add(albumCommentPanel, "South");

		return albumInfoPanel;
	}

	/** 
	 * Scrolls the scrollpane acording to mouse wheel rotation
	 * @param e The mouse wheel event to process
	 */
	protected void processMouseWheelEvent(MouseWheelEvent e) {
		int scrollUnits = 40;
		
		Point oldPosition = getScrollPosition();

		Point newPosition = new Point();
		newPosition.setLocation(
			oldPosition.getX(),
			oldPosition.getY() + e.getWheelRotation()*scrollUnits);
		
		setScrollPosition(newPosition);
		return;
	}
	
	/**
	 * Adds a new album row containing the given album info.
	 * If album is null, an empty row is inserted at the top.
	 * 
	 * @param album The album to add
	 */
	public void addAlbumRowInfo(Album album, int insertIndex) {
		String albumName = "";
		boolean publish = false;
		boolean privateOnly = false;
		String year = "";
		String month = "";
		String country = "";
		String comment = "";
		
		if(album != null) {
			albumName = album.getAlbumName();
			publish = new Boolean(album.getPublish()).booleanValue();
			privateOnly = new Boolean(album.getPrivateOnly()).booleanValue();
			year = album.getYear();
			month = album.getMonth();
			country = album.getCountry();
			comment = album.getComment();
		}

		//Create album name panel info components
		TextField albumNameField = new TextField(albumName, 40);
		Checkbox publishBox = new Checkbox("Publish    ");
		publishBox.setFont(new Font("Arial", Font.BOLD, 12));
		publishBox.setState(publish);
		Checkbox privateOnlyBox = new Checkbox("Password protected");
		privateOnlyBox.setFont(new Font("Arial", Font.BOLD, 12));
		privateOnlyBox.setState(privateOnly);
	
		//Create album year, month and country info components
		TextField yearField = new TextField(year, 4);
		TextField monthField = new TextField(month, 12);
		TextField countryField = new TextField(country, 12);
		
		//Create and add comment text area panel info components
		TextField commentField = new TextField(80);
		commentField.setText(comment);

		//Create up and down buttons
		Button upButton = new Button(Actions.EAP_BUTTON_MOVE_UP);
		upButton.addActionListener(this);
		Button removeButton = new Button(Actions.EAP_BUTTON_REMOVE);
		removeButton.addActionListener(this);
		Button downButton = new Button(Actions.EAP_BUTTON_MOVE_DOWN);
		downButton.addActionListener(this);
		Button toTopButton = new Button(Actions.EAP_BUTTON_TO_TOP);
		toTopButton.addActionListener(this);
		Button insertNewButton = new Button(Actions.EAP_BUTTON_INSERT_NEW);
		insertNewButton.addActionListener(this);
		Button toBottomButton = new Button(Actions.EAP_BUTTON_TO_BOTTOM);
		toBottomButton.addActionListener(this);
	
		//Add album info components to vectors for later retrieval
		if(insertIndex == -1) {
			//Insert at bottom
			insertIndex = albumNameFields.size();
		}

		albumNameFields.insertElementAt(albumNameField, insertIndex);
		publishBoxes.insertElementAt(publishBox, insertIndex);
		privateOnlyBoxes.insertElementAt(privateOnlyBox, insertIndex);
		yearFields.insertElementAt(yearField, insertIndex);
		monthFields.insertElementAt(monthField, insertIndex);
		countryFields.insertElementAt(countryField, insertIndex);
		commentFields.insertElementAt(commentField, insertIndex);		
		upButtons.insertElementAt(upButton, insertIndex);		
		removeButtons.insertElementAt(removeButton, insertIndex);
		downButtons.insertElementAt(downButton, insertIndex);
		toTopButtons.insertElementAt(toTopButton, insertIndex);		
		insertNewButtons.insertElementAt(insertNewButton, insertIndex);
		toBottomButtons.insertElementAt(toBottomButton, insertIndex);
	}
		
	/**
	 * Adds album rows from photo descriptors. Looks through all descriptors for
	 * photos to be published that have album names not allready in the list.
	 * 
	 */
	public void addAlbumsFromPhotos() {
		theController.setStatus("Adding albums from published photos.");
		
		String[] photoDescPaths = theController.getDescriptorPaths();

		int numOfAlbumsAdded = 0;
		//Loop through all photo descriptors
		for(int descIndex = 0; descIndex < photoDescPaths.length; descIndex++) {

			String descPath = new File(photoDescPaths[descIndex]).getParentFile().getPath();

			Vector photos = null;
			try {
				photos = theController.readPhotoDescriptor(descPath);
			} catch(AtacamaException exception) {
				//Failed to read a descriptor, drop it and continue
				continue;
			}
					
			//Loop through all photos in descriptor
			for(int photoIndex = 0; photoIndex < photos.size(); photoIndex++) {
				Photo photo = (Photo) photos.elementAt(photoIndex);
				
				boolean albumExists = false;
				//Loop through all albums
				for(int albumIndex = 0; albumIndex < albumNameFields.size(); albumIndex++) {
					String albumName = ((TextField) albumNameFields.elementAt(albumIndex)).getText().trim();
					
					//If album 
					if(!albumName.equals("") &&
						albumName.equalsIgnoreCase(photo.getAlbumName())) {
							albumExists = true;
						}
				}
				
				if(!albumExists && new Boolean(photo.getPublish()).booleanValue()) {
					//We found an album name in the photos that does not exist
					//in the album rows, and the corresponding photo is to be published
					Album newAlbum = new Album();
					newAlbum.setAlbumName(photo.getAlbumName());
					
					//Insert new album at top
					addAlbumRowInfo(newAlbum, 0);
					
					numOfAlbumsAdded++;
				}
			}
		}
				
		theController.setStatus("Added " + numOfAlbumsAdded + " albums from published photos.");
	}

	/**
	 * Moves and element from one position to another in a vector
	 * 
	 * @param vector The vector to move the element for
	 * @param index1 The initial index of element
	 * @param index2 The final index of the element
	 */
	private void moveElement(Vector vector, int index1, int index2) {
		vector.insertElementAt(vector.remove(index1), index2);
	}
		
	/**
	 * Switches order of two elements in all vectors
	 * 
	 * @param oldIndex
	 * @param newIndex
	 */
	private void moveElementForAll(int oldIndex, int newIndex) {
		//Switch vector elements order
		moveElement(albumNameFields, oldIndex, newIndex);
		moveElement(publishBoxes, oldIndex, newIndex);
		moveElement(privateOnlyBoxes, oldIndex, newIndex);
		moveElement(yearFields, oldIndex, newIndex);
		moveElement(monthFields, oldIndex, newIndex);
		moveElement(countryFields, oldIndex, newIndex);
		moveElement(commentFields, oldIndex, newIndex);
		moveElement(upButtons, oldIndex, newIndex);
		moveElement(removeButtons, oldIndex, newIndex);
		moveElement(downButtons, oldIndex, newIndex);		
	}
	
	/**
	 * Repaints the entire scroll pane with values from properties and descriptors
	 * @see java.awt.Component#repaint()
	 */
	public void repaint() {
		//Repaint the scroll pane
		removeAll();
		add(initRowsPanel());
		theController.repaintMainFrame();			
	}
		
	/**
	 * The action peformed method
	 * @param event The action event to process
	 */
	public void actionPerformed(ActionEvent event) {
		//Destinguish between up and down buttons
		int upIndex = upButtons.indexOf(event.getSource());
		int downIndex = downButtons.indexOf(event.getSource());
		int removeIndex = removeButtons.indexOf(event.getSource());
		int toTopIndex = toTopButtons.indexOf(event.getSource());
		int insertNewIndex = insertNewButtons.indexOf(event.getSource());
		int toBottomIndex = toBottomButtons.indexOf(event.getSource());

		int oldIndex = -1;
		int newIndex = -1;
		
		if(upIndex > 0) {
			//An up button has been pressed, and it is not the top button
			oldIndex = upIndex;
			newIndex = upIndex - 1 ;
			moveElementForAll(oldIndex, newIndex);
		} else if (downIndex > -1 && downIndex < albumNameFields.size() - 1) {
			//A down button has been pressed, and it is not the bottom button
			oldIndex = downIndex;
			newIndex = downIndex + 1 ;
			moveElementForAll(oldIndex, newIndex);
		} else if(removeIndex > -1) {
			//A remove button has been pressed, remove the entry from all vectors
			albumNameFields.removeElementAt(removeIndex);
			publishBoxes.removeElementAt(removeIndex);
			privateOnlyBoxes.removeElementAt(removeIndex);
			yearFields.removeElementAt(removeIndex);
			monthFields.removeElementAt(removeIndex);
			countryFields.removeElementAt(removeIndex);
			commentFields.removeElementAt(removeIndex);		
			upButtons.removeElementAt(removeIndex);		
			removeButtons.removeElementAt(removeIndex);
			downButtons.removeElementAt(removeIndex);
			repaint();
		} else if(toTopIndex > -1) {
			//A to top button has been pressed, move the entry to top
			//A down button has been pressed, and it is not the bottom button
			oldIndex = toTopIndex;
			newIndex = 0;			
			moveElementForAll(oldIndex, newIndex);
		} else if(insertNewIndex > -1) {
			//Insert new empty row at desired index
			addAlbumRowInfo(null, insertNewIndex);
		} else if(toBottomIndex > -1) {
			//A to bottom button has been pressed, move the entry to bottom
			//A down button has been pressed, and it is not the bottom button
			oldIndex = toBottomIndex;
			newIndex = albumNameFields.size() - 1;			
			moveElementForAll(oldIndex, newIndex);
		}
		
		//Repaint the scroll pane
		repaint();		
	}
}
