/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.log.Logger;

/**
 * @author ramon.arellano
 *
 * This is the main panel in the generate gallery mode. It gives functionality
 * for genererating HTML and photos/thumbs for by reading album and photo descriptors.
 * 
 * The panel also gives functionality for viewing the generated gallery in the user
 * default browser.
 */
public class GenerateGalleryPanel extends Panel implements ActionListener {

	//Controller, Config, Logger and FileManager	
	private static Controller theController = null;
	private static Config theConfig = null;
	private static Logger theLogger = null;

	//Components
	private Label photosPathLabel = null;
	private Label albumDescriptorLabel = null;
	private Label targetPathLabel = null;
	private Checkbox htmlGenerateBox = null;
	private Checkbox htmlOverwriteBox = null;
	private TextField photosPerRowField = null;
	private Checkbox photoGenerateBox = null;
	private Checkbox photoOverwriteBox = null;
	private TextField albumThumbField = null;
	private TextField photoThumbField = null;
	private TextField photoField = null;
	private TextField qualityField = null;

	/**
	 * The Constructor
	 */
	public GenerateGalleryPanel() {
		//Get the controller, config, logger and file manager
		theController = Controller.getInstance();
		theConfig = Config.getInstance();
		theLogger = theLogger.getInstance();

		//Initialise the panel
		init();
	}

	/**
	 * Initialises the edit album descriptor panel with all components
	 */
	public void init() {
		//Might be refreshing
		removeAll();

		//Set layout for edit album descriptor panel		 
		setLayout(new BorderLayout());

		//Add options panel to edit descriptors panel
		add(initPathsPanel(), "North");

		//Add scroll pane with all albums from descriptor
		add(initOptionsPanel(), "Center");

		//Add log panel
		add(initButtonsAndLogPanel(), "South");

		theController.repaintMainFrame();
	}

	/**
	 * Initializes the buttons and log panel
	 * 
	 * @return The buttons and log panel
	 */
	private Panel initButtonsAndLogPanel() {
		//Create components for button panel
		Button saveSettingsButton = new Button(Actions.GG_BUTTON_SAVE_OPTIONS);
		saveSettingsButton.addActionListener(this);
		saveSettingsButton.setFont(new Font("Arial", Font.BOLD, 12));

		Button generateButton = new Button(Actions.GG_BUTTON_GENERATE);
		generateButton.addActionListener(this);
		generateButton.setFont(new Font("Arial", Font.BOLD, 12));

		Button viewGalleryButton = new Button(Actions.GG_BUTTON_VIEW_GALLERY);
		viewGalleryButton.addActionListener(this);
		viewGalleryButton.setFont(new Font("Arial", Font.BOLD, 12));

		Button compareFlickrButton = new Button(Actions.GG_BUTTON_COMPARE_FLICKR);
		compareFlickrButton.addActionListener(this);
		compareFlickrButton.setFont(new Font("Arial", Font.BOLD, 12));

		Button syncFlickrButton = new Button(Actions.GG_BUTTON_SYNC_FLICKR);
		syncFlickrButton.addActionListener(this);
		syncFlickrButton.setFont(new Font("Arial", Font.BOLD, 12));

		//Disactivate view button if album index file doesn't exists
		File albumIndexFile =
			new File(
				theConfig.getString("common.dir.target.gallery")
					+ theConfig.getString("html.album.index"));
		if (!albumIndexFile.exists()) {
			viewGalleryButton.setEnabled(false);
		}

		//Create buttons panel and add components
		Panel buttonsPanel = new Panel(new FlowLayout(FlowLayout.RIGHT));
		buttonsPanel.add(saveSettingsButton);
		buttonsPanel.add(generateButton);
		buttonsPanel.add(viewGalleryButton);
		buttonsPanel.add(compareFlickrButton);
		buttonsPanel.add(syncFlickrButton);

		//Create buttonsAndLogPanel and add components
		Panel buttonsAndLogPanel = new Panel(new BorderLayout());
		buttonsAndLogPanel.add(buttonsPanel, "Center");
		if(!LogView.logFrameIsOpen()) {
			buttonsAndLogPanel.add(LogView.getInstance().getPanel(), "South");
		}
		
		return buttonsAndLogPanel;
	}

	/**
	 * Initializes and return the paths panel
	 * 
	 * @return The paths panel
	 */
	private Panel initPathsPanel() {

		//Create photos path selection panel
		photosPathLabel =
			new Label(theConfig.getString("common.dir.source.photos"));
		Panel selectPhotosPathPanel =
			new FileSelectionPanel(
				"Photos directory:  ",
				photosPathLabel,
				Actions.COMMON_BUTTON_CHANGE_PHOTO_DIR,
				this);

		//Create album descriptor file selection panel
		albumDescriptorLabel =
			new Label(theConfig.getString("descriptor.file.text.album"));
		Panel selectAlbumDescriptorPanel =
			new FileSelectionPanel(
				"Album descriptor:",
				albumDescriptorLabel,
				Actions.GG_BUTTON_CHANGE_ALBUM_DESCRIPTOR,
				this);

		//Create target path selection panel
		targetPathLabel =
			new Label(theConfig.getString("common.dir.target.gallery"));
		Panel selectTargetPathPanel =
			new FileSelectionPanel(
				"Target directory:   ",
				targetPathLabel,
				Actions.GG_BUTTON_CHANGE_TARGET_PATH,
				this);

		//Create album directory selection panel and add components
		Panel pathsPanel = new Panel();
		pathsPanel.setLayout(new GridLayout(3, 1));
		pathsPanel.add(selectPhotosPathPanel);
		pathsPanel.add(selectAlbumDescriptorPanel);
		pathsPanel.add(selectTargetPathPanel);

		return pathsPanel;
	}

	/**
	 * Initializes and return the options panel
	 * 
	 * @return The options panel
	 */
	private Panel initOptionsPanel() {
		//HTML options label
		Panel htmlLabelPanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		Label htmlLabel = new Label("HTML OPTIONS");
		htmlLabel.setFont(new Font("Arial", Font.BOLD, 12));
		htmlLabelPanel.add(htmlLabel);

		//HTML overwrite
		Panel htmlOverwritePanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		htmlGenerateBox =
			new Checkbox(
				"Generate HTML                                         ",
				theConfig.getBool("html.generate"));
		htmlOverwriteBox =
			new Checkbox(
				"Overwrite existing HTML files",
				theConfig.getBool("html.overwrite"));
		htmlOverwritePanel.add(htmlGenerateBox);
		htmlOverwritePanel.add(htmlOverwriteBox);

		//HTML Photos per row
		Panel htmlPhotosPerRowPanel =
			new Panel(new FlowLayout(FlowLayout.LEFT));
		htmlPhotosPerRowPanel.add(
			new Label("Number of thumbnails per row in index "));
		photosPerRowField = new TextField(2);
		photosPerRowField.setText(
			theConfig.getString("html.design.noframes.photos.per.row"));
		htmlPhotosPerRowPanel.add(photosPerRowField);

		//Photos options label
		Panel photoLabelPanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		Label photoLabel = new Label("PHOTO AND THUMBNAIL OPTIONS");
		photoLabel.setFont(new Font("Arial", Font.BOLD, 12));
		photoLabelPanel.add(photoLabel);

		//Photo overwrite
		Panel photoOverwritePanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		photoGenerateBox =
			new Checkbox(
				"Generate photos and thumbnails         ",
				theConfig.getBool("photo.generate"));
		photoOverwriteBox =
			new Checkbox(
				"Overwrite existing photo files",
				theConfig.getBool("photo.overwrite"));
		photoOverwritePanel.add(photoGenerateBox);
		photoOverwritePanel.add(photoOverwriteBox);

		//Photo album thumnail size
		Panel albumThumbPanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		albumThumbPanel.add(
			new Label("Album thumbnail max height (pixels)     "));
		albumThumbField = new TextField(5);
		albumThumbField.setText(
			theConfig.getString("photo.thumb.album.height"));
		albumThumbPanel.add(albumThumbField);

		//Photo thumnail size
		Panel photoThumbPanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		photoThumbPanel.add(
			new Label("Photo thumbnail max height (pixels)      "));
		photoThumbField = new TextField(5);
		photoThumbField.setText(theConfig.getString("photo.thumb.height"));
		photoThumbPanel.add(photoThumbField);

		//Photo size
		Panel photoPanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		photoPanel.add(
			new Label("Photo max height (pixels)                         "));
		photoField = new TextField(5);
		photoField.setText(theConfig.getString("photo.height"));
		photoPanel.add(photoField);

		//Photo size
		Panel qualityPanel = new Panel(new FlowLayout(FlowLayout.LEFT));
		qualityPanel.add(
			new Label("JPEG quality (between 0.0 - 1.0)             "));
		qualityField = new TextField(5);
		qualityField.setText(theConfig.getString("photo.jpeg.quality"));
		qualityPanel.add(qualityField);

		//Add all panels to options panel
		Panel optionsPanel = new Panel(new GridLayout(10, 1));
		optionsPanel.add(htmlLabelPanel);
		optionsPanel.add(htmlOverwritePanel);
		optionsPanel.add(htmlPhotosPerRowPanel);
		optionsPanel.add(new Label(" "));
		optionsPanel.add(photoLabelPanel);
		optionsPanel.add(photoOverwritePanel);
		optionsPanel.add(albumThumbPanel);
		optionsPanel.add(photoThumbPanel);
		optionsPanel.add(photoPanel);
		optionsPanel.add(qualityPanel);

		return optionsPanel;
	}

	/**
	 * Update properties from panel
	 */
	public void updateProperties() {
		theConfig.setProperty(
			"html.generate",
			Boolean.toString(htmlGenerateBox.getState()));
		theConfig.setProperty(
			"html.overwrite",
			Boolean.toString(htmlOverwriteBox.getState()));
		theConfig.setProperty(
			"html.design.noframes.photos.per.row",
			photosPerRowField.getText());
		theConfig.setProperty(
			"photo.generate",
			Boolean.toString(photoGenerateBox.getState()));
		theConfig.setProperty(
			"photo.overwrite",
			Boolean.toString(photoOverwriteBox.getState()));
		theConfig.setProperty(
			"photo.thumb.album.height",
			albumThumbField.getText());
		theConfig.setProperty("photo.thumb.height", photoThumbField.getText());
		theConfig.setProperty("photo.height", photoField.getText());
		theConfig.setProperty("photo.jpeg.quality", qualityField.getText());

	}

	/**
	 * Receives events from the buttons in the panel
	 * @param event The action event
	 */
	public void actionPerformed(ActionEvent event) {
		String action = event.getActionCommand();


		try {
			if (action.equalsIgnoreCase(Actions.COMMON_BUTTON_CHANGE_PHOTO_DIR)) {
				//The change photos path button was pushed
				//Open window to select new file
				File newFile =
					theController.selectDirectory(
						"Select photos path",
						new File(theConfig.getString("common.dir.source.photos")));

				//If file was changed we need to update descriptor scroll pane
				if (newFile != null) {
					//Set properties (update model)
					theConfig.setProperty(
						"common.dir.source.photos",
						newFile.getPath());

					//Update view
					photosPathLabel.setText(newFile.getPath());
				} else {
					//No file selected
					new UserDialog("No photo directory was selected.");
				}
			} else if (
				action.equalsIgnoreCase(
					Actions.GG_BUTTON_CHANGE_ALBUM_DESCRIPTOR)) {
				//The change album file button was pushed
				//Open window to select new file
				File newFile =
					theController.selectFile(
						"Select album file",
						new File(theConfig.getString("descriptor.file.text.album")),
						false);

				//If file was changed we need to update descriptor scroll pane
				if (newFile != null) {
					//Set properties (update model)
					theConfig.setProperty(
						"descriptor.file.text.album",
						newFile.getPath());

					//Update view
					albumDescriptorLabel.setText(newFile.getPath());
				} else {
					//No file selected
					new UserDialog("No album descriptor file was selected.");
				}
			} else if (
				action.equalsIgnoreCase(Actions.GG_BUTTON_CHANGE_TARGET_PATH)) {
				//The change target path button was pushed
				//Open window to select new file
				File newFile =
					theController.selectDirectory(
						"Select target directory",
						new File(theConfig.getString("common.dir.target.gallery")));

				//If file was changed we need to update descriptor scroll pane
				if (newFile != null) {
					//Set properties (update model)
					theConfig.setProperty(
						"common.dir.target.gallery",
						newFile.getPath());

					//Update view
					init();
				} else {
					//No file selected
					new UserDialog("No gallery target directory was selected.");
				}
			} else if (action.equalsIgnoreCase(Actions.GG_BUTTON_SAVE_OPTIONS)) {
				//The change save setting button was pushed
				//Update the properties
				updateProperties();

				//Save properties to file
				theController.saveProperties();
			} else if (action.equalsIgnoreCase(Actions.GG_BUTTON_GENERATE)) {
				//Update the properties
				updateProperties();

				//Save properties to file
				theController.saveProperties();

				theController.setStatus("Generating gallery...");
				//The generate gallery button has been pushed
				//Generate gallery
				theController.generateGallery(
					htmlGenerateBox.getState(),
					photoGenerateBox.getState());

				theController.setStatus("Finished generating gallery");
				init();
			} else if (action.equalsIgnoreCase(Actions.GG_BUTTON_VIEW_GALLERY)) {
				//The view gallery in browser button was pushed
				theController.openGalleryInDefaultBrowser();
			} else if (action.equalsIgnoreCase(Actions.GG_BUTTON_COMPARE_FLICKR)) {
				//Run synchronization with logOnly flag set to true
				theController.compareFlickr();
			} else if (action.equalsIgnoreCase(Actions.GG_BUTTON_SYNC_FLICKR)) {
				//Run synchronization with logOnly flag set to false
				theController.syncFlickr();
			}
		} catch (RuntimeException e) {
			theLogger.logError("Runtime excpetion", e);
		}
	}
}