/*
 * Created on 09.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * @author ramon.arellano
 *
 * This class displays a log using the text area log created in the main frame.
 */
public class LogView implements WindowListener, TextListener {

	//Components
	private TextArea logArea = null;

	//The unique instance of LogView
	private static LogView theLogView = null;

	//The log frame instance
	private Frame logFrame = null;

	//The unique instance of the log panel
	private Panel logPanel = null;
	
	//The controller
	Controller theController = null;
	
	/**
	 * The constructor
	 */
	private LogView() {
		theController = Controller.getInstance();
		logPanel = new Panel();
		initLogPanel();
	}


	/**
	 * Returns the unique instance of this class
	 * 
	 * @return The unique instance of this class
	 */
	public static LogView getInstance() {
		if(theLogView == null) {
			theLogView = new LogView();
		}
		
		return theLogView;
	}


	/**
	 * Constructor specifying parent blank frame
	 */
	public void openInFrame() {
		if(logFrame == null) {
			logFrame = new Frame();
			logFrame.setTitle("Atacama Log");
			logFrame.addWindowListener(theLogView);
			theController.locateCentered(theLogView.logFrame, 
				Constants.LOG_FRAME_DIMENSION);
			logFrame.add(getPanel());
		}
		
		theLogView.logFrame.show();
	}


	/**
	 * Returns the log panel
	 * 
	 * @return The log panel
	 */
	public Panel getPanel() {
		return logPanel;
	}
	
	
	/**
	 * Checks if log frame is already open
	 * 
	 * @return True if open
	 */
	public static boolean logFrameIsOpen() {
		if(LogView.getInstance().logFrame != null) {
			return true;
		} else {
			return false;
		}
		
	}
	
	/**
	 * Initializes the log panel
	 */
	private void initLogPanel() {
			
		//Add label
		Label logLabel = new Label("  Log:");
		logLabel.setFont(new Font("Arial", Font.BOLD, 12));
		
		//Add log area
		logArea = Controller.getInstance().getLogArea();
		logArea.setEditable(false);
		logArea.setFont(new Font("Courier", Font.PLAIN, 11));
		logArea.addTextListener(this);	
			
		//Add components
		logPanel = new Panel();
		logPanel.setLayout(new BorderLayout());		
		logPanel.add(logLabel, "North");
		logPanel.add(logArea, "Center");	
	}


	/**
	 * The log changed. Scroll to bottom.
	 * @param event The text event
	 */
	public void textValueChanged(TextEvent event) {
		if(event.getSource() == logArea) {
			//This trails the log
			logArea.select(1000000, 1000000);
		}
	}

	//Implementation of  all window listener interface methods
	public void windowClosing(WindowEvent event) { 
		logFrame.dispose();
		logFrame = null;
	} 
	public void windowClosed(WindowEvent event)  {} //do nothing for now 
	public void windowDeiconified(WindowEvent event){} 
	public void windowIconified(WindowEvent event){} 
	public void windowActivated(WindowEvent event){} 
	public void windowDeactivated(WindowEvent event){} 
	public void windowOpened(WindowEvent event){}

}
