/*
 * Created on 07.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import com.arellano.ramon.atacama.config.Config;

/**
 * @author ramon.arellano
 * A frame for displaying images
 *
 */
public class PhotoPreviewFrame extends Frame implements WindowListener {
	//The thumbnail image
	private Image resizedImage;

	/**
	 * Constructor
	 * 
	 * @param imageFilePath Image file path
	 */
	public PhotoPreviewFrame(String imageFilePath) {

		setTitle(imageFilePath);
		Image image = getToolkit().getImage(imageFilePath);

		//Scale to correct size
		resizedImage =
			image.getScaledInstance(
				-1,
				Config.getInstance().getInt("photo.height"),
				Image.SCALE_SMOOTH);

		//Wait for the image to be loaded
		MediaTracker mediaTracker = new MediaTracker(this);
		mediaTracker.addImage(resizedImage, 0);
		try {
			mediaTracker.waitForID(0);
		} catch (InterruptedException ie) {
			System.err.println(ie);
			System.exit(1);
		}

		//Set the canvas to image size
		setSize(resizedImage.getWidth(null), resizedImage.getHeight(null));
		setLocation(100, 100);

		addWindowListener(this);
		show();
	}

	/** 
	 * Paint method
	 * @param graphics The graphics
	 */
	public void paint(Graphics graphics) {
		if (resizedImage != null) {
			graphics.drawImage(resizedImage, 0, 0, this);
		}
	}

	//Implementation of  all window listener interface methods
	public void windowClosing(WindowEvent event) {
		dispose();
	}
	public void windowClosed(WindowEvent event) {
	} //do nothing for now 
	public void windowDeiconified(WindowEvent event) {
	}
	public void windowIconified(WindowEvent event) {
	}
	public void windowActivated(WindowEvent event) {
	}
	public void windowDeactivated(WindowEvent event) {
	}
	public void windowOpened(WindowEvent event) {
	}
}
