/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Point;
import java.awt.ScrollPane;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseWheelEvent;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 * The scroll pane for displaying photo descriptors. Shows all info of the
 * photos including a thumbnail created from the image file.
 */
public class PhotoScrollPane extends ScrollPane
	implements ActionListener, ItemListener {

	//Controller, Config, Logger and FileManager	
	private static Controller theController = null;
	private static Config theConfig = null;

	//The photos and album vector
	public Vector photos = null;
	private Vector albums = null;

	//Components
	public Vector publishBoxes = null;
	public Vector primaryBoxes = null;
	public Vector albumNameFields = null;
	public Vector commentFields = null;
	public Vector viewButtons = null;
	public Vector albumChoices = null;

	/**
	 * The constructor
	 */
	public PhotoScrollPane() {
		//Get the controller, config, logger and file manager
		theController = Controller.getInstance();
		theConfig = Config.getInstance();		
	}

	/**
	 * Initialises the panel with all components
	 * @param descriptorPath The descriptor path
	 */
	public void init(String descriptorPath) {	
		//Might be refreshing, removel all first
		removeAll();

		//Get albums from album descriptor (to be used in album choice)
		albums = theController.readAlbumDescriptor(
				theConfig.getString("descriptor.file.text.album"), false);
		
		//Get the photos for the specified descriptor
		try {
			photos = theController.readPhotoDescriptor(descriptorPath);
		} catch(AtacamaException exception) {
			new UserDialog(exception.getMessage());
			return;
		}
		
		//Instantiate vectors
		publishBoxes = new Vector(photos.size());
		primaryBoxes = new Vector(photos.size());
		albumNameFields = new Vector(photos.size());
		commentFields = new Vector(photos.size());
		viewButtons =  new Vector(photos.size());
		albumChoices =  new Vector(photos.size());
		
		//Create new rows planel
		Panel rowsPanel = new Panel();

		//Set rowspanel to same number of rows as photo vector
		GridLayout gridLayout = new GridLayout();
		gridLayout.setColumns(3);
		gridLayout.setRows(photos.size());
		rowsPanel.setLayout(gridLayout);

		//Add rows to panel for each photo in descriptor		
		for (int index = 0; index < photos.size(); index++) {
			//Get corresponding photo
			Photo photo = (Photo) photos.elementAt(index);

			//Create add columns panel
			Panel columnsPanel = new Panel();
			columnsPanel.setLayout(new BorderLayout());
			ThumbnailCanvas thumbCanvas = theController.getThumbnailCanvas(photo.getPhotoFilePath());
			columnsPanel.add(thumbCanvas, "Center");
			columnsPanel.add(initPhotoInfoPanel(photo), "East");

			//Alternate color of rows
			if(new Float(index).floatValue()/2 - (index/2) == 0.5) {
				columnsPanel.setBackground(Color.gray);
			}

			//Add columns panel to row panel
			rowsPanel.add(columnsPanel);			
		}

		//Add rows panel to scroll pane
		add(rowsPanel);
	}

	/**
	 * Creates a panel with the information for a single photo
	 * 
	 * @param photo The photo to be shown
	 * @return The photo info panel
	 */
	private Panel initPhotoInfoPanel(Photo photo) {
		//Create photo name panel components
		Label photoLabel = new Label("Photo:       ");
		photoLabel.setFont(new Font("Arial", Font.BOLD, 12));
		Label photoNameLabel = new Label(photo.getPhotoName());
		photoNameLabel.setFont(new Font("Arial", Font.PLAIN, 12));
		Checkbox publishBox = new Checkbox("Publish    ");
		publishBox.setState(new Boolean(photo.getPublish()).booleanValue());
		publishBox.setFont(new Font("Arial", Font.BOLD, 12));
		Button viewButton = new Button(Actions.EDP_BUTTON_VIEW_PHOTO);
		viewButton.addActionListener(this);
		Checkbox primaryBox = new Checkbox("Primary    ");
		primaryBox.setState(new Boolean(photo.getPrimary()).booleanValue());
		primaryBox.setFont(new Font("Arial", Font.BOLD, 12));

		//Create and add photo name panel
		Panel photoNamePanel = new Panel();
		photoNamePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		photoNamePanel.add(photoLabel);
		photoNamePanel.add(photoNameLabel);
		photoNamePanel.add(publishBox);
		photoNamePanel.add(viewButton);
		photoNamePanel.add(primaryBox);

		//Create and add photo album and publish panel components
		Label albumLabel = new Label("Album:      ");
		albumLabel.setFont(new Font("Arial", Font.BOLD, 12));
		TextField albumField = new TextField(photo.getAlbumName().trim(), 18);
		Label albumChoiceLabel = new Label("  Select existing album:");
		albumChoiceLabel.setFont(new Font("Arial", Font.BOLD, 12));
		Choice albumChoice = initAlbumChoice();

		//Create and add photo album and publish panel
		Panel photoAlbumPanel = new Panel();
		photoAlbumPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		photoAlbumPanel.add(albumLabel);
		photoAlbumPanel.add(albumField);
		photoAlbumPanel.add(albumChoiceLabel);
		photoAlbumPanel.add(albumChoice);

		//Create and add comment text area panel components
		Label commentLabel = new Label("Comment:");
		commentLabel.setFont(new Font("Arial", Font.BOLD, 12));
		TextField commentField = new TextField(80);
		commentField.setText(photo.getComment().trim());

		//Create and add comment text area panel
		Panel photoCommentPanel = new Panel();
		photoCommentPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		photoCommentPanel.add(commentLabel);
		photoCommentPanel.add(commentField);

		//Create photo info panel
		Panel photoInfoPanel = new Panel();
		photoInfoPanel.setLayout(new BorderLayout());
		photoInfoPanel.add(photoNamePanel, "North");
		photoInfoPanel.add(photoAlbumPanel, "Center");
		photoInfoPanel.add(photoCommentPanel, "South");

		//Add photo info components to vector for later retrieval
		publishBoxes.add(publishBox);
		primaryBoxes.add(primaryBox);
		albumNameFields.add(albumField);
		commentFields.add(commentField);
		viewButtons.add(viewButton);

		return photoInfoPanel;
	}


	/**
	 * Creates a choice element with all album names in the album  descriptor
	 * 
	 * @return The choice component containing album names
	 */
	private Choice initAlbumChoice() {
		//Create new choice component and add to vector
		Choice albumChoice = new Choice();
		albumChoice.addItemListener(this);
		albumChoices.add(albumChoice);
		
		//The first choice should always be empty to avoid confusing the user
		albumChoice.add("                      ");
				
		if(albums == null || albums.size() < 1) {
			//No albums to add to choice
			albumChoice.setEnabled(false);
			return albumChoice;
		}
		
		//Add all existing non-empty album names from album descriptor file 
		boolean addedAlbum = false;
		for(int index = 0; index < albums.size(); index++) {
			String albumName = ((Album)albums.elementAt(index)).getAlbumName();
			
			if(albumName != null && !albumName.trim().equals("")) {
				albumChoice.addItem(albumName);
				addedAlbum = true;
			}
		}
		
		if(!addedAlbum) {
			albumChoice.setEnabled(false);
		}
		
		return albumChoice;
	}
	
	
	/** 
	 * Sets the scrolling to a specified number of units
	 * @param e The mouse wheel event
	 */
	protected void processMouseWheelEvent(MouseWheelEvent e) {
		int scrollUnits = 40;
		
		Point oldPosition = getScrollPosition();

		Point newPosition = new Point();
		newPosition.setLocation(
			oldPosition.getX(),
			oldPosition.getY() + e.getWheelRotation()*scrollUnits);
		
		setScrollPosition(newPosition);
		return;
	}
	
	/**
	 * Handles action events for this panel
	 * @param event The action performed
	 */
	public void actionPerformed(ActionEvent event) {
		//Will assume it is a view button while we only have these button on the panel
		int index = viewButtons.indexOf(event.getSource());
		
		Photo photo = (Photo) photos.elementAt(index);
		
		theController.viewPhoto(photo.getPhotoFilePath());
	}
	
	/**
	 * Listens for changes in the album choice components
	 * @param event The item event
	 */
	public void itemStateChanged(ItemEvent event) {
		//Find out which album choice component was changed
		int index = albumChoices.indexOf(event.getSource());
		
		Choice albumChoice = (Choice) albumChoices.elementAt(index);
		
		//Set the album name field for this photo to the chosen existing album name
		TextField albumNameField = (TextField) albumNameFields.elementAt(index);
		albumNameField.setText(albumChoice.getSelectedItem().trim());
	}	
}
