/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Arrays;

/**
 * @author Ayusya
 *
 * Class for letting user select a directory. Originally created by a Japanes guy.
 */
public class SelectDirectoryDialog {
	protected static final String ST_OK = " OK ";
	protected static final String ST_CANCEL = " CANCEL";
	protected static final String ST_UP = " UP ";

	Dialog dialog = null;
	File file = null;
	Choice drive = null;
	List list = null;
	TextField text = null;

	/**
	 *
	 */
	class OnEvent
		extends WindowAdapter
		implements ActionListener, ItemListener {

		/**
		 * @param e
		 */
		public void windowClosing(WindowEvent e) {
			onCancel();
		}

		/**
		 * @param e
		 */
		public void actionPerformed(ActionEvent e) {
			String st = e.getActionCommand();
			if (st.equals(ST_OK))
				onOK();
			else if (st.equals(ST_CANCEL))
				onCancel();
			else if (st.equals(ST_UP))
				onUp();
		}

		/**
		 * @param e
		 */
		public void itemStateChanged(ItemEvent e) {
			if (e.getSource() == drive)
				onDrive();
		}
	}

	/**
	 * The constructor
	 * 
	 * @param parent
	 * @param msg
	 * @param title
	 * @param current
	 */
	protected SelectDirectoryDialog(
		Frame parent,
		String msg,
		String title,
		File current) {

		dialog = new Dialog(parent, true);
		dialog.setLayout(new BorderLayout());
		dialog.setTitle(title);
		dialog.setLocation(200, 200);
		dialog.resize(500, 250);

		if ((file = current) == null) {
			file = new File(".").getAbsoluteFile().getParentFile();
		}

		Button b;
		OnEvent event = new OnEvent();
		dialog.addWindowListener(event);

		Panel north = new Panel(), south = new Panel();
		north.setLayout(new BorderLayout());
		dialog.add(north, BorderLayout.NORTH);
		dialog.add(south, BorderLayout.SOUTH);

		north.add(new Label(msg, Label.CENTER), BorderLayout.NORTH);
		drive = new Choice();
		drive.addItemListener(event);
		File[] roots = File.listRoots();
		for (int i = 0; i < roots.length; i++) {
			drive.add(roots[i].getPath());
		}
		drive.select(file.getAbsolutePath().substring(0, 3));
		north.add(drive, BorderLayout.WEST);
		text = new TextField(file.getAbsolutePath(), 32);
		text.setEnabled(false);
		north.add(text, BorderLayout.CENTER);
		b = new Button(ST_UP);
		b.addActionListener(event);
		north.add(b, BorderLayout.EAST);

		list = new List();
		dialog.add(list, BorderLayout.CENTER);
		list.addMouseListener(new OnMouse());

		b = new Button(ST_OK);
		b.addActionListener(event);
		south.add(b);
		b = new Button(ST_CANCEL);
		b.addActionListener(event);
		south.add(b);

		updateList();
	}

	/**
	 * 
	 */
	protected void onDrive() {
		String drv = drive.getSelectedItem();
		if (drv == null)
			return;
		//		System.out.println("drive.item:" + drv);
		file = new File(drv);
		updateList();
	}

	class OnMouse extends MouseAdapter {
		/**
		 * @param e
		 */
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2) {
				Object o = e.getSource();
				if (o == list)
					onList();
			}
		}
	}

	/**
	 * 
	 */
	protected void onList() {
		String st = list.getSelectedItem();
		if (st == null)
			return;
		String path = file.getAbsolutePath();
		if (path.length() > 3)
			path += File.separator;
		file = new File(path + st);
		updateList();
	}

	/**
	 * 
	 */
	protected void onCancel() {
		dialog.hide();
		file = null;
	}

	/**
	 * 
	 */
	protected void onOK() {
		String st = list.getSelectedItem();
		if (st != null) {
			if (file.toString().length() == 3)
				file = new File(file.toString() + st);
			else
				file = new File(file.toString() + File.separator + st);
		}
		dialog.hide();
	}

	/**
	 * 
	 */
	protected void onUp() {
		File file2 = file.getAbsoluteFile().getParentFile();
		if (file2 != null) {
			file = file2;
			updateList();
		}
	}

	/**
	 * 
	 */
	protected void updateList() {
		list.removeAll();
		File[] files = file.listFiles();

		if (files == null)
			return;

		Arrays.sort(files);
		text.setText(file.getAbsolutePath());
		for (int i = 0; i < files.length; i++) {
			if (files[i].isDirectory()) {
				list.add(files[i].getName());
			}
		}
	}

	/**
	 * @param parent
	 * @param msg
	 * @param title
	 * @param current
	 * @return the file
	 */
	public static File show(
		Frame parent,
		String msg,
		String title,
		File current) {
		SelectDirectoryDialog me =
			new SelectDirectoryDialog(parent, msg, title, current);
		return me.showDialog();
	}

	/**
	 * @return the file
	 */
	private File showDialog() {
		dialog.show();
		return file;
	}
}