/*
 * Created on 07.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.Button;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.arellano.ramon.atacama.config.Config;

/**
 * @author ramon.arellano
 * The atacam splash window
 *
 */
public class SplashWindow extends Window implements ActionListener {

	//The background image
	Image image = null;
	
	//The config and controller
	Config theConfig = null;
	Controller theController = null;
	
	/**
	 * Constructor. Displays the splash window.
	 * 
	 */
	public SplashWindow() {
		super(Controller.getInstance().getMainFrame());
		
		theConfig = Config.getInstance();
		theController = Controller.getInstance();
		
		//Read background image
		image = getToolkit().getImage(theConfig.getApplicationDir()
			+ theConfig.getString("gui.file.image.splash"));

		//Wait for the image to be loaded
		MediaTracker mediaTracker = new MediaTracker(this);
		mediaTracker.addImage(image, 0);
		try {
			mediaTracker.waitForID(0);
		} catch (InterruptedException ie) {
			System.err.println(ie);
			System.exit(1);
		}

		//Place the splash window in the middle of the screen
		theController.locateCentered(this, new Dimension(image.getWidth(null), image.getHeight(null)));
		
		//Create and add buttons to layout
		setLayout(null);
				
		Button readManualButton = new Button(Actions.SPLASH_READ_MANUAL);
		readManualButton.setBounds(180, 320, 140, 25);
		readManualButton.setFont(new Font("Arial", Font.BOLD, 12));
		readManualButton.addActionListener(this);
		Button startButton = new Button(Actions.SPLASH_START);
		startButton.setFont(new Font("Arial", Font.BOLD, 12));
		startButton.setBounds(340, 320, 140, 25);
		startButton.addActionListener(this);
		
		add(readManualButton);
		add(startButton);
		show();
	}

	/** 
	 * Paint method
	 * @param graphics The graphics
	 */
	public void paint(Graphics graphics) {
		if (image != null) {
			graphics.drawImage(image, 0, 0, this);
		}

		String credit1 = "The Atacama Gallery Generator";
		String credit2 = "designed and written by Ramon Kristian Arellano.";

		graphics.setFont(new Font("Arial", Font.BOLD, 13));
		graphics.drawString(credit1, 20,90); 	
		graphics.drawString(credit2, 20,120); 	
	}
	
	/**
	 * Handles the window's button events
	 * @param event The event
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {
		String action = event.getActionCommand();
		
		if(action.equals(Actions.SPLASH_READ_MANUAL)) {
			dispose();
			theController.getMainFrame().setEnabled(true);
			theController.getMainFrame().initManualDialog();
		} else if(action.equals(Actions.SPLASH_START)) {
			dispose();
			theController.getMainFrame().setEnabled(true);
		}
	}
}
