/*
 * Created on 06.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.io.File;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.log.Logger;

/**
 * Class for displaying thumbnail images
 */
class ThumbnailCanvas extends Component {
	//The thumbnail image
	private Image thumbnailImage;

	/**
	 * Constructor
	 */
	public ThumbnailCanvas() {
	}
	
	/**
	 * Constructor
	 * 
	 * @param imageFilePath Image file path
	 * @param resize True if image should be resized
	 */
	public ThumbnailCanvas(String imageFilePath, boolean resize) {
		
		//Check that image exists
		if(!new File(imageFilePath).exists()) {
			Logger.getInstance().logWarning("Could not create thumb because file '"
				+ imageFilePath
				+ "' does not exist.");
		}
		
		Image image = getToolkit().getImage(imageFilePath);

		if(resize) {
			//Scale to correct size
			thumbnailImage = image.getScaledInstance(
				-1,
				Config.getInstance().getInt("photo.thumb.album.height"),
				Image.SCALE_FAST);
		} else {
			//Don't scale
			thumbnailImage = image;
		}
		
		//Wait for the image to be loaded
		MediaTracker mediaTracker = new MediaTracker(this);
		mediaTracker.addImage(thumbnailImage, 0);
		try {
			mediaTracker.waitForID(0);
		} catch (InterruptedException ie) {
			System.err.println(ie);
			System.exit(1);
		}

		//Set the canvas to image size
		resize(thumbnailImage.getWidth(null), thumbnailImage.getHeight(null));
	}

	/** 
	 * Paint method
	 * @param graphics The graphics
	 */
	public void paint(Graphics graphics) {
		if (thumbnailImage != null) {
			graphics.drawImage(thumbnailImage, 0, 0, this);
		}
	}
}
