/*
 * Created on 07.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * @author ramon.arellano
 *
 * A class for displaying user messages. Mostly used when an exception is thrown.
 */
public class UserDialog extends Dialog implements WindowListener {

	//The controller instance
	Controller theController = null;

	/**
	 * The Constructor for warning messages
	 * The Constructor
	 * @param message
	 */
	public UserDialog(String message) {
		//	Calls the parent telling it this
		//	dialog is modal(i.e true)
		super(Controller.getInstance().getMainFrame(), true);

		initUserDialog("Warning", message, Constants.WARNING_DIALOG_DIMENSION, false);

	}

	/**
	 * Constructor that let's the user decide the dialog fram size and title
	 * @param message The message
	 * @param size The dialog size
	 * @param scrollbars True if scrollbars should be displayed
	 */
	public UserDialog(
		String title,
		String message,
		Dimension size,
		boolean scrollbars) {

		super(Controller.getInstance().getMainFrame(), true);

		initUserDialog(title, message, size, scrollbars);
	}

	
	/**
	 * Initializes the dialog frame
	 * 
	 * @param title
	 * @param message
	 * @param size The dialog size
	 * @param scrollbars
	 */
	private void initUserDialog(
		String title,
		String message,
		Dimension size,
		boolean scrollbars) {

			theController = Controller.getInstance();
			
			setTitle(title);
			setLayout(new BorderLayout());

			int scroll = 0;
			if(scrollbars) {
				scroll = TextArea.SCROLLBARS_VERTICAL_ONLY;
			} else {
				scroll = TextArea.SCROLLBARS_NONE;
			}

			TextArea messageArea =
				new TextArea(message, 3, 3, scroll);
			messageArea.setEditable(false);

			Button okButton = new Button(Actions.UD_BUTTON_OK);

			Panel dialogPanel = new Panel(new BorderLayout());
			dialogPanel.add(messageArea, "Center");
			dialogPanel.add(okButton, "South");

			theController.locateCentered(this, size);
			add(dialogPanel);
			addWindowListener(this);

			show();
		}
			
	/**
	 * Closes the window when the ok button is pressed
	 * 
	 * @param evt The event
	 * @param arg The object
	 * @return Unknown
	 * @see java.awt.Component#action(java.awt.Event, java.lang.Object)
	 */
	public boolean action(Event evt, Object arg) {
		dispose();
		return true;
	}

	//Implementation of  all window listener interface methods
	//Implementation of  all window listener interface methods
	public void windowClosing(WindowEvent event) {
		dispose();
	}
	public void windowClosed(WindowEvent event) {
	} //do nothing for now 
	public void windowDeiconified(WindowEvent event) {
	}
	public void windowIconified(WindowEvent event) {
	}
	public void windowActivated(WindowEvent event) {
	}
	public void windowDeactivated(WindowEvent event) {
	}
	public void windowOpened(WindowEvent event) {
	}
}
