/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.photo;

import java.io.File;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 * 
 * Copies and resizes photos for the album
 */
public class PhotoGenerator {

	//Logger, Config and FileManager
	static private Config theConfig = null;
	static private Logger theLogger = null;
	static private FileManager theFileManager = null;

	/**
	 * Constructor
	 */
	public PhotoGenerator() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * This is the main method that copies and resizes all photos to
	 * their corresponding gallery directories.
	 * 
	 * @param albums The vector of albums
	 */
	public void generate(Vector albums) throws AtacamaException {		
		
		//Read necessary properties only once
		int largeImageHeight = theConfig.getInt("photo.height");
		int thumbHeight = theConfig.getInt("photo.thumb.height");
		int albumThumbHeight = theConfig.getInt("photo.thumb.album.height");
		
		//Loop through all galleries and create thumb and images for
		//all photos.
		ImageManipulator imageResizer = new ImageManipulator();
		for(int albumIndex=0; albumIndex < albums.size(); albumIndex++) {
			Album album = (Album) albums.elementAt(albumIndex);

			//Check if there are photos to resize
			if(album.getPhotos() == null || album.getPhotos().size() == 0) {
				theLogger.logWarning("No photos to resize for album '"
					+ album.getAlbumName()
					+ "'.");
				continue;
			}

			//Make photo directories for this album if they don't already exists
			String albumDir = theConfig.getAlbumPhotosDir(album, false);
			if(!new File(albumDir).exists()) {
				//Album photo and thumb directories do not exist, create them
				new File(albumDir
					+ theConfig.getString("photo.dir.target.photo.large")).mkdirs();

				new File(albumDir
					+ theConfig.getString("photo.dir.target.photo.thumb")).mkdirs();
			}

			//Create album thumb destination file
			File albumThumbFile = new File(albumDir
				+ theConfig.getString("photo.image.album.thumb"));

			//Create album thumb using primary photo
			Photo primaryPhoto = (Photo) album.getPrimaryPhoto();
			//Check if photo is JPG
			if(theFileManager.isFileType(new File(primaryPhoto.getPhotoName()), "JPG")) {
				String primaryPhotoFilePath = primaryPhoto.getPhotoFilePath();
				File primaryPhotoFile = new File(primaryPhotoFilePath);
				if(!albumThumbFile.exists() || theConfig.getBool("photo.overwrite")) {
					imageResizer.resizeAndCropSquare(primaryPhotoFile, albumThumbFile, albumThumbHeight);
				}
			}
			
			//Loop through all photos for this album
			for(int photoIndex=0; photoIndex<album.getPhotos().size(); photoIndex++) {
				Photo photo = (Photo) album.getPhotos().elementAt(photoIndex);
				
				String photoFilePath = photo.getPhotoFilePath();
				File photoFile = new File(photoFilePath);
								
				//Create large photo destination file
				File largePhotoFile = new File(albumDir
					+ theConfig.getString("photo.dir.target.photo.large")
					+ photo.getPhotoName());
				
				//Check if photo is JPG
				if(!theFileManager.isFileType(new File(photo.getPhotoName()), "JPG")) {
					//It's not JPG. Copy file instead of resizing
					theFileManager.copyFile(photoFile, largePhotoFile);

					//Do not need to create album and photo thumbs for non JPG files					
					continue;
				}

				//Create photo thumb destination file
				File thumbFile = new File(albumDir
					+ theConfig.getString("photo.dir.target.photo.thumb")
					+ photo.getPhotoName());
				
				//Create large image if allowed
				//(does not resize if file is lower then specified max height)
				if(!largePhotoFile.exists() || theConfig.getBool("photo.overwrite")) {				
					theLogger.logTrace("Resizing album: '"
						+ album.getAlbumName() 
						+ "' photo: '" 
						+ photo.getPhotoName() 
						+ "'");
					imageResizer.resize(photoFile, largePhotoFile, largeImageHeight);
				}
				
				//Create thumb only if allowed
				if(!thumbFile.exists() || theConfig.getBool("photo.overwrite")) {
					imageResizer.resizeAndCropSquare(photoFile, thumbFile, thumbHeight);
				}				
			}
		}
	}
}
