/*
 * Created on May 28, 2004
 *
 */
package com.arellano.ramon.atacama.system;

import java.io.File;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;

public class Tester {
	protected static String applicationPath = null;
	protected static String testPath = null;
	protected static String testPhotosPath = null;
	protected static String testPhotoDirectory = null;
	protected static String testPhotoDirectory2 = null;
	protected static String testResultsPath = null;
	protected static String testExpectedPath = null;
	
	static protected Config theConfig = null;
	static protected Logger theLogger = null;
	static protected FileManager theFileManager = null;	


	public Tester() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();

		//Create paths
		applicationPath = theConfig.getApplicationDir();
		testPath = applicationPath + theConfig.getString("test.dir.testing");
		testPhotosPath = theConfig.getString("common.dir.source.photos");
		testPhotoDirectory = testPath + theConfig.getString("test.dir.photo.directory");
		testPhotoDirectory2 = testPath + theConfig.getString("test.dir.photo.directory2");
		testResultsPath = testPath + theConfig.getString("test.dir.results");
		testExpectedPath = testPath + theConfig.getString("test.dir.expected");
	}
	
	protected void appendResult(StringBuffer resultString, String message) {
		resultString.append(message + "\n");
	}
	
	protected void writeResultFile(StringBuffer resultString, boolean writeToLog) throws AtacamaException {
		Throwable throwable =  new Throwable();
		StackTraceElement stackTraceElement = throwable.getStackTrace()[1];
		String resultFileName = stackTraceElement.getMethodName().toLowerCase() + ".txt";

		theFileManager.writeTextFile(new File(testResultsPath, resultFileName), resultString.toString(), true);
		
		if(writeToLog) {
			theLogger.logTrace("\n" + resultString.toString());
		}
	}
	
	protected void validateTest() throws AtacamaException {
		Throwable throwable =  new Throwable();
		StackTraceElement stackTraceElement = throwable.getStackTrace()[1];
		String resultFileName = stackTraceElement.getMethodName().toLowerCase() + ".txt";

		//Check that the results file exist
		if(!new File(testResultsPath, resultFileName).exists()) {
			theLogger.logError("Could not validate test because results file was not found: " + new File(testResultsPath, resultFileName).getPath());
		}

		//Check that the expected results file exist
		if(!new File(testExpectedPath, resultFileName).exists()) {
			theLogger.logError("Could not validate test because expected results file was not found: " + new File(testExpectedPath, resultFileName).getPath());
		}

		int resultCode = theFileManager.compareTextFiles(new File(testResultsPath, resultFileName), new File(testExpectedPath, resultFileName));
		if(resultCode == 0) {
			theLogger.logTrace("PASSED: " + stackTraceElement.getMethodName());
		} else if(resultCode == -1) {
			theLogger.logTrace("FAILED: " + stackTraceElement.getMethodName() + " : Files have different number of lines");			
		} else {
			theLogger.logTrace("FAILED: " + stackTraceElement.getMethodName() + " : Differences in line " + resultCode);			
		}
	}
}
