/*
 * InterestingPeopleServlet.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr.example.servlet;

import java.io.*;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.*;
import javax.servlet.http.*;
import org.jickr.Flickr;
import org.jickr.FlickrException;
import org.jickr.Photo;
import org.jickr.Photo.PhotoSize;
import org.jickr.PhotoSearch;
import org.jickr.User;

/**
 * <P>
 * Example servlet.  I've put it in the doc directory (unlike other examples)
 * so that I don't have to worry about building it.
 * <P>
 * This example gets a username, then displays the most interesting public
 * photos for the user.
 * <P>
 * Don't forget that your server security policy needs to be changed to allow
 * network access by your servlet.  Personally, I prefer to just turn security
 * off.
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 *
 */
public class InterestingPeopleServlet extends HttpServlet {
    
    private static final int NUMPHOTOS = 50;
    
    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        
        String username = request.getParameter("username");
        if (username == null) username = "";
        
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter out = response.getWriter();
        
        out.println("<html>");
        out.println("<head>");
        out.println("<title>The Interesting People Servlet</title>");
        out.println("</head>");
        out.println("<body>");
        out.println("<h1>Find Interesting People</h1>");
        if (username == "") {
            out.println("<P>Enter the username below, and press OK.  This page will");
            out.println("go out to Flickr and get the top "+NUMPHOTOS+" interesting photos for that user.");
            out.println("<P>Please be patient - it can take a minute.");
        } 
        out.println("<P><form name=\"Username Input Form\" action=\""+request.getRequestURI()+"\">");
        out.println("Enter the username: <input type=\"text\" name=\"username\" value=\""+username+"\" />");
        out.println("<input type=\"submit\" value=\"OK\" />");
        
        if (username != "") {
            try {
                Flickr.setApiKey("invalidkey");
                Flickr.setSharedSecret("invalidsecret");
                User user = User.findByUsername(username);
                String realname = user.getRealName();
                out.println("<a href=\""+user.getProfileURL()+"\">");
                if (realname.equals("")) {
                    out.println("Found User ID: "+user.getID());
                } else { // we have a real name
                    out.println("Found User Name: "+user.getRealName());
                }
                out.println("</a>");
                out.println("<BR><BR>");
                out.println("<HR>");
                
                PhotoSearch search = new PhotoSearch();
                search.setUser(user);
                search.setSort(PhotoSearch.Sort.INTERESTING_DESC);
                List photos = Photo.search(search);
                Iterator i = photos.iterator();
                int count = 0;
                while (i.hasNext() && count++ < NUMPHOTOS) {
                    Photo photo = (Photo)i.next();
                    Map sizes = photo.getSizes();
                    Photo.PhotoSize thumb = (PhotoSize) sizes.get(Photo.Size.THUMB);
                    if (thumb != null) {
                        out.println("<a href=\""+photo.getPhotoPage()+"\"> ");
                        out.println("<img src=\""+thumb.getStaticURL()+"\" alt=\""+
                                photo.getTitle()+"\">");
                        out.println("</a>");
                    }
                }
            } catch (FlickrException fe) {
                if (fe.getCode() == 1) {
                    out.println("User not found");
                } else {
                    out.println("Error accessing Flickr<BR>");
                    if (fe.getCode() >0 ) out.println("Error Code is: "+fe.getCode()+"<BR>");
                    fe.printStackTrace(out);
                }
            }
            
        } else {
            out.println("<BR>");
            out.println("<HR>");
        }
        
        
        out.println("</body>");
        out.println("</html>");
        
        out.close();
    }
    
    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Short description";
    }
    // </editor-fold>
}
