/*
 * FlickrException.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

/**
 * Generic exception thrown by Jickr objects.  This exception is thrown primarily if 
 * an error is returned from Flickr, or Flickr returns unexpected data.  Coding errors
 * typically throw a <code>FlickrRuntimeException</code>.  Please note that this
 * exception has the notion of an error code, which corresponds to the error code returned
 * by Flickr on any error.
 * @see FlickrRuntimeException
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class FlickrException extends java.lang.Exception {

    private int code = 0;
    
    /**
     * Creates a new instance of <code>FlickrException</code> without detail 
     * message.  Made private because we don't really want that, after all.
     */
    private  FlickrException() {
    }

    /**
     * Constructs an instance of <code>FlickrException</code> with the specified
     * detail message.
     * @param msg the detail message.
     */
    public FlickrException(String msg) {
        super(msg);
    }
    
    /**
     * Constructs an instance of <code>FlickrException</code> with the specified
     * detail message.
     * @param msg the detail message.
     * @param code The error code for the message.  This corresponds to the Flickr Error code. See the <a href="http://www.flickr.com/services/api/">Flickr API documentation</a>.
     * 
     */
    public FlickrException(String msg, int code) {
        super(msg);
        this.code = code;
    }
    
    /**
     * Constructs an instance of <code>FlickrException</code> with the specified
     * detail message, and the root cause exception.
     * @param msg the detail message.
     * @param cause the root cause of the exception
     */
    public FlickrException(String msg, Throwable cause) {
        super(msg,cause);
    }

    /**
     * If the FlickrException is returned because of a Flickr Error, then this 
     * code is set to be the same as that returned by Flickr.  See the 
     * <a href="http://www.flickr.com/services/api/">Flickr API documentation</a>
     * for more information. This code is 0 if not set by Flickr.
     * @return code Flickr return code.
     */
    public int getCode() {
        return code;
    }
}