/*
 * Permission.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

/**
 * Authentication Permission allowed.  They are ordered as: READ, WRITE, DELETE.  Higher permissions
 * also allow access to lower permissions.
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public enum Permission {
    /**
     * READ permission allows the program to look at private photos that the 
     * authenticated user has permission to see on Flickr.
     */
    READ, 
    /**
     * WRITE permission allows the program to modify data about photos on Flickr,
     * if the authenticated user has permission to do it on Flickr.
     */
    WRITE, 
    /**
     * DELETE permission allows the program to delete photos, if the authenticated
     * user has permission to do it on Flickr.
     */
    DELETE}
