/*
 * FlickrThumbs.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr.example;

import java.util.List;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ListIterator;
import javax.swing.*;
import org.jickr.Flickr;
import org.jickr.FlickrException;
import org.jickr.Photo;
import org.jickr.PhotoSearch;
import org.jickr.User;

/**
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class FlickrThumbs extends JFrame implements ActionListener {
    JPanel thumbPanel;
    JScrollPane scrollPane;
    ListIterator<Photo> li;
    int count = 0;
    // Start #, Maximum # of pictures to display.
    final int STARTNUM = 0;
    final int MAXNUM = 120;
    
    /**
     * Creates a new instance of FlickrThumbs
     */
    public FlickrThumbs(List<Photo> list) {
        super("Flickr Thumbnails");
        JFrame.setDefaultLookAndFeelDecorated(true);
        li = list.listIterator(STARTNUM);
        setSize(650,500);
        
        thumbPanel = new JPanel(new GridLayout(0,6));
        scrollPane = new JScrollPane(thumbPanel,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        getContentPane().add(scrollPane);
        thumbPanel.setBackground(Color.BLACK);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        new Timer(0, this).start();
        setVisible(true);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws FlickrException {

        try {
            BufferedReader br;
            br = new BufferedReader(new FileReader("keys"));
            
            String apiKey = br.readLine();
            Flickr.setApiKey(apiKey);
            String sharedSecret = br.readLine();
            Flickr.setSharedSecret(sharedSecret);
        } catch (IOException ex) {
            ex.printStackTrace();
            System.exit(1);
        } finally {
        }
        
        /*  The Following block is the ususal one to include in programs.
        String apiKey = "notavalidapikey";
        Flickr.setApiKey(apiKey);
         
        String sharedSecret = "notavalidsecretkey";
        Flickr.setSharedSecret(sharedSecret);
         */
        
        System.out.println("Getting Photo List");
        List<Photo> list = Photo.getInteresting();
        //PhotoSearch search = new PhotoSearch();
        //search.setUser(User.findByUsername("netdance"));
        //search.setSort(PhotoSearch.Sort.INTERESTING_DESC);
        //List<Photo> list = Photo.search(search);
        new FlickrThumbs(list);
    }
    
    public void actionPerformed(ActionEvent ae) {
        if (!li.hasNext()) return;  //we're done
        if (++count > MAXNUM) return; //also done
        try {
            Photo photo = li.next();
            System.out.println("Getting Photo "+photo.getID());
            JLabel imageLabel = new JLabel(new ImageIcon(photo.getImage(Photo.Size.THUMB)));
            thumbPanel.add(imageLabel);
        } catch (FlickrException ex) {
            System.out.println(ex.getMessage());
        }
        thumbPanel.revalidate();
        repaint();
    }
}
