/*
 * LoginTest.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr.example;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import org.jickr.*;
/**
 * A class to demonstrate how to do Authentication.
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class LoginTest {
    
    /**
     *
     * Creates a new instance of LoginTest
     */
    public static void main(String[] args) throws Exception {
        
        // We'll read the API Key and Shared Secret from a file, since
        // We don't want to check in our keys.
        // Normally, we'd use the next block of code and hardcode the keys instead

        BufferedReader br = new BufferedReader(new FileReader("keys"));
        String apiKey = br.readLine();
        Flickr.setApiKey(apiKey);
        String sharedSecret = br.readLine();
        Flickr.setSharedSecret(sharedSecret);

        /*
        //  The Following block is the ususal one to include in programs.
        String apiKey = "notavalidapikey";
        Flickr.setApiKey(apiKey);
         
        String sharedSecret = "notavalidsharedsecret";
        Flickr.setSharedSecret(sharedSecret);
         */
        
        Permission perm = Permission.DELETE; // access level we want
        User user = Auth.getDefaultAuthUser(); // Check to see if we've already authenticated
        //User user = User.findByUsername("netdance"); // or look for a specific user
        
        if (user != null && Auth.isAuthenticated(user, perm)) {
            System.out.println("The Default Authenticated User is \""+user.getUserName()+"\"");
            System.out.println("Already authenticated to at least "+perm+" level.");
            System.out.println("We're actually at the "+Auth.getPermLevel(user)+" level.");
            Auth.setAuthContext(user);
            System.out.println("We've used "+Auth.getBandwidthUsed()+
                    " bytes bandwidth out of "+Auth.getBandwidthMax()+
                    " bytes available.");
            System.out.println("We have room for "+Auth.getFilesizeMax()+
                    " bytes of files.");
            System.exit(0);
        }
        // Give the URL to enter in the browser
        System.out.println("Please enter the following URL into a browser, " +
                "follow the instructions, and then come back");
        String authURL = Auth.getAuthURL(perm);
        System.out.println(authURL);
        // Wait for them to come back
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        in.readLine();
        // Alright, now that we're cleared with Flickr, let's try to authenticate
        System.out.println("Trying to get "+perm+" access.");
        try {
            user = Auth.authenticate();
        } catch (FlickrException ex) {
            System.out.println("Failed to authenticate.");
            ex.printStackTrace(System.out);
            System.exit(1);
        }
        if (Auth.isAuthenticated(user,perm)) {
            Auth.setDefaultAuthUser(user);
            Auth.setAuthContext(user);
            System.out.println("We've authenticated user \""+user.getUserName()+
                    "\" with "+Auth.getPermLevel(user)+" access.");
            System.out.println("We've used "+Auth.getBandwidthUsed()+
                    " bytes bandwidth out of "+Auth.getBandwidthMax()+
                    " bytes available.");
            System.out.println("We have room for "+Auth.getFilesizeMax()+
                    " bytes of files.");
            Auth.setDefaultAuthUser(user);
        } else {
            //Shouldn't ever get here - we throw an exception above if we can't authenticate.
            System.out.println("Oddly unauthenticated");
        }
    }
}

