/*
 * PhotoInfo.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr.example;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.List;
import org.jickr.*;

/**
 *
 * @author driscoll
 */
public class PhotoInfo {
    
    /** Creates a new instance of PhotoInfo */
    public PhotoInfo() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
        
        // We'll read the API Key and Shared Secret from a file, since
        // We don't want to check in our keys.
        // Normally, we'd use the next block of code and hardcode the keys instead
        
        BufferedReader br = new BufferedReader(new FileReader("keys"));
        String apiKey = br.readLine();
        Flickr.setApiKey(apiKey);
        String sharedSecret = br.readLine();
        Flickr.setSharedSecret(sharedSecret);
        
        /*
        //  The Following block is the ususal one to include in programs.
        String apiKey = "notavalidapikey";
        Flickr.setApiKey(apiKey);
         
        String sharedSecret = "notavalidsharedsecret";
        Flickr.setSharedSecret(sharedSecret);
         */
        
        User user = User.findByUsername("netdance"); // Set to whatever user you like.
        outputList(user.getPublicPhotos(),25);
        
    }
    
    private static void outputList(List<Photo> photolist, int maxnum) throws Exception {
        System.out.println("---------------------------------------------");
        
        int count = 0;
        for (Photo photo : photolist) {
            if (count++ >= maxnum) break;
            System.out.println("Photo Tile: "+photo.getTitle());
            System.out.println("Photo Desc: "+photo.getDescription());
            System.out.println("Photo Uploaded: "+photo.getDateUploaded());
            System.out.println("Photo Taken: "+photo.getDateTaken());
            System.out.println("Photo Taken Granularity: "+photo.getDateTakenGranularity());
            System.out.println("Photo Updated: "+photo.getDateUpdated());
            System.out.println("Photo ID: "+photo.getID());
            System.out.println("Photo Secret: "+photo.getSecret());
            System.out.println("Photo Comment #: "+photo.getNumComments());
            for (Comment comment : photo.getComments()) {
                System.out.println("    Photo Comment ID: "+comment.getID());
                System.out.println("    Photo Comment Author: "+comment.getAuthor().getUserName());
                System.out.println("    Photo Comment Text: "+comment.getText());
            }
            System.out.println("Photo Page: "+photo.getPhotoPage());
            System.out.println("Photo URL: "+photo.getStaticURL());
            System.out.println("Photo License: "+photo.getLicense().getName());
            for (Tag tag : photo.getTags()) {
                System.out.println("    Photo Tag: "+tag.getText());
            }
            for (Note note : photo.getNotes()) {
                System.out.println("    Photo Note ID: "+note.getID());
                System.out.println("    Photo Note Author: "+note.getAuthor());
                System.out.println("    Photo Note Area: "+note.getX()+" "+note.getY()+" "+note.getWidth()+" "+note.getHeight());
                System.out.println("    Photo Note Text: "+note.getText());
            }
            for (Group group : photo.getPublicGroups()) {
                System.out.println("    Group ID: "+group.getID());
                System.out.println("    Group Name: "+group.getName());
                System.out.println("    Group # Members: "+group.getNumMembers());
            }
            for (PhotoSet set : photo.getPublicSets()) {
                System.out.println("    Set ID: "+set.getID());
                System.out.println("    Set Title: "+set.getTitle());
                System.out.println("    Set Description: "+set.getDescription());
                System.out.println("    Set # Photos: "+set.getNumPhotos());
            }
            
            System.out.println("---------------------------------------------");
        }
        
    }
    
}
