/*
 * TagAdd.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr.example;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.List;
import org.jickr.*;

/**
 *
 * @author driscoll
 */
public class TagAdd {
    
    /** Creates a new instance of TagAdd */
    public TagAdd() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
        
        /*
         * This code assumes a file called "keys" in the working directory,
         * of the format apikey\nshared\n
         * I did this so I wouldn't have to check in my api key into the
         * workspace.
         */
        BufferedReader br;
        br = new BufferedReader(new FileReader("keys"));
        String apiKey = br.readLine();
        Flickr.setApiKey(apiKey);
        String sharedSecret = br.readLine();
        Flickr.setSharedSecret(sharedSecret);
        
        /*  The Following block is the ususal one to include in programs.
        String apiKey = "notavalidapikey";
        Flickr.setApiKey(apiKey);
         
        String sharedSecret = "notavalidsecretkey";
        Flickr.setSharedSecret(sharedSecret);
         */
        
        User authUser = Auth.getDefaultAuthUser();
        if (!Auth.isAuthenticated(authUser,Permission.WRITE)) throw new Exception("Unauthenticated");
        Auth.setAuthContext(authUser);
        
        String id = "insert photo id here";
        String secret = "insert photo secret # here";
        
        Photo photo = new Photo(id,secret);
        
        List<Tag> taglist = photo.getTags();
        for (Tag tag : taglist) {
            System.out.println(tag.getText());
        }
        
        photo.addTags("newtesttag");
        
        taglist = photo.getTags();
        for (Tag tag : taglist) {
            System.out.println(tag.getText());
            if (tag.getText().equals("newtesttag")) tag.remove();
        }

        taglist = photo.getTags();
        for (Tag tag : taglist) {
            System.out.println(tag.getText());
        }
        
    }
    
}
